import re
import warnings
from urllib.parse import urlencode
import requests
from swagger_parser import SwaggerParser


class KialiSwaggerParser:

    def __init__(self, swagger_address, custom_base_path=None):
        json_object = requests.get(swagger_address).json()

        if custom_base_path is not None:
            json_object['basePath'] = custom_base_path + json_object['basePath']
            
        self.swagger = SwaggerParser(swagger_dict=json_object)


    def construct_url(self, operation, path=None, params=None):
        try:
            base_url = self.swagger.operation.get(operation)[0]
        except TypeError:
            raise Exception('Name not found on Swagger File')

        if path is not None:
            for key, value in path.items():
                base_url = re.sub("{" + key + "}", value, base_url)

        if params is not None:
            base_url = base_url + '?' + urlencode(params)

        return base_url




#!/usr/bin/python
from kiali import KialiClient
def main(): 
    module = AnsibleModule(
        argument_spec=dict(
            kiali_api_hostname= dict(
                default=os.environ.get('KIALI_HOSTNAME'), type='str'),
            
            kiali_api_port= dict(
                default=os.environ.get('KIALI_PORT'), type='int'),
            
            kiali_api_username=dict(
                default=os.environ.get('KIALI_USERNAME'), type='str'),

            kiali_api_password=dict(
                default=os.environ.get('KIALI_PASSWORD'), type='str'),

            
            kiali_api_swagger_address=dict(
                default=os.environ.get('KIALI_SWAGGER_ADDRESS'), type='str'),
                
            
            kiali_api_scheme= dict(required=False, type='str', choices=['https', 'http'], default='https'),

            kiali_api_auth_type = dict(required=False, type='str', choices=['https-user-password', 'no-auth'], default='https-user-password'),
            
            kiali_api_request= dict(required=True, type='dict'),

            
            kiali_api_verify_ssl=dict(required=False, type='bool', default=False),
        )
    )

    for arg in ['kiali_api_hostname', 'kiali_api_port', 'kiali_api_username', 'kiali_api_password']:
        if module.params[arg] in (None, ''):
            module.fail_json(msg="missing required argument: {}".format(arg))

    hostname       = module.params['kiali_api_hostname']
    port       = module.params['kiali_api_port']
    username   = module.params['kiali_api_username']
    password      = module.params['kiali_api_password']
    scheme  = module.params['kiali_api_scheme']
    auth_type  = module.params['kiali_api_auth_type']
    verify_ssl = module.params['kiali_api_verify_ssl']
    request = module.params['kiali_api_request']
    
    
    try:
        params = request['params']
    except KeyError:
          params = None

    try:
        path = request['path']
    except KeyError:
          path = None

    print(params)

    kiali_client = KialiClient(hostname=hostname, scheme=scheme, port=port, auth_type=auth_type, username=username, password=password, verify=verify_ssl)
    response = kiali_client.request(method_name=request['method_name'], path=path,  params=params)

    full_response = {'status_code': response.status_code, 'url': response.url, 'text': response.text,'elapsed_time': response.elapsed.total_seconds()}

    module.exit_json(**full_response)


# Import module bits
from ansible.module_utils.basic import *
if __name__ == "__main__":
    main()


from kiali.api_connector import KialiHTTPSApiConnector
from kiali.api_connector import KialiNoAuthApiConnector
from kiali.swagger import KialiSwaggerParser

class KialiClient():
    def __init__(self, hostname='localhost', scheme='https', port='443', auth_type='https-user-password', username='admin', password='admin', verify=False, swagger_address='https://raw.githubusercontent.com/kiali/kiali/master/swagger.json', custom_base_path=None, max_retries=5):
        self.swagger_parser = KialiSwaggerParser(swagger_address=swagger_address, custom_base_path=custom_base_path)

        # TODO Add Oauth Connector
        if auth_type == 'https-user-password':
            self.api_connector = KialiHTTPSApiConnector(hostname=hostname, scheme=scheme, port=port, verify=verify, username=username, password=password, max_retries=max_retries)


        if auth_type == 'no-auth':
            self.api_connector = KialiNoAuthApiConnector(hostname=hostname, scheme=scheme, port=port, verify=verify)

    # Facade Pattern
    def request(self, method_name=None, path=None, params=None, plain_url=None, http_method='GET', data=None):

        if plain_url is None:
            url = self.swagger_parser.construct_url(method_name, path, params)
            return self.api_connector.dispatcher(url=url, http_method=http_method, data=data)
        else:
            return self.api_connector.dispatcher(url=plain_url, params=params, http_method=http_method, data=data)




from __future__ import unicode_literals

from abc import ABC
import urllib3
import requests
import requests.utils
from requests.auth import HTTPBasicAuth


class KialiApiConnector(ABC):

    def __init__(self, hostname, port, scheme, verify, auth, max_retries):
        self.hostname = hostname
        self.port = port
        self.scheme = scheme
        self.auth = auth
        self.verify = verify
        self.max_retries = max_retries

    def retrieve_url(self, path):
        return requests.utils.urlunparse((self.scheme, (self.hostname + ":" +str(self.port)), path, None, None, None))

    def create_session(self):
        session = requests.Session()
        session.auth = self.auth
        requests.adapters.HTTPAdapter(max_retries=self.max_retries)
        session.headers.update({'Host': self.hostname, 'Content-Type': 'application/json'})
        return session

    # Factory Method of HTTP Request
    def dispatcher(self, url, params=None, http_method='GET', data=None):
        if http_method is 'GET':
            return self.get(url=url, params=params)

        if http_method is 'POST':
            return self.post(url=url, params=params, data=data)

        if http_method is 'PATCH':
            return self.patch(url=url, params=params, data=data)

        if http_method is 'DELETE':
            return self.delete(url=url, params=params, data=data)


    def get(self, url, params=None):
        session = self.create_session()
        return session.get(url=self.retrieve_url(url), verify=self.verify, params=params)

    def post(self, url, data, params=None):
        session = self.create_session()
        return session.post(url=self.retrieve_url(url), verify=self.verify, data=data, params=params)

    def patch(self, url, data, params=None):
        session = self.create_session()
        return session.patch(url=self.retrieve_url(url), verify=self.verify, data=data, params=params)

    def delete(self, url, data, params=None):
        session = self.create_session()
        return session.delete(url=self.retrieve_url(url), verify=self.verify, data=data, params=params)


class KialiHTTPSApiConnector(KialiApiConnector):
    """
    Creates new client for Kiali based on HTTPS
    """

    def __init__(self, hostname, port, scheme, username, password, verify, max_retries):

        super().__init__(hostname=hostname, port=port, scheme=scheme,
                         auth=HTTPBasicAuth(username, password), verify=verify, max_retries=max_retries)


class KialiNoAuthApiConnector(KialiApiConnector):
    """
        Creates new client for Kiali based on HTTP
        """

    def __init__(self, hostname, port, scheme, verify):
        super().__init__(hostname=hostname, port=port, scheme=scheme,
                         auth=None, verify=verify)

